@@ -1,15 +1,15 @@
-# 
+#
 # This file is part of Test-Pod-LinkCheck
-# 
-# This software is copyright (c) 2010 by Apocalypse.
-# 
+#
+# This software is copyright (c) 2011 by Apocalypse.
+#
 # This is free software; you can redistribute it and/or modify it under
 # the same terms as the Perl 5 programming language system itself.
-# 
+#
 use strict; use warnings;
 package Test::Pod::LinkCheck;
 BEGIN {
-  $Test::Pod::LinkCheck::VERSION = '0.004';
+  $Test::Pod::LinkCheck::VERSION = '0.005';
 }
 BEGIN {
   $Test::Pod::LinkCheck::AUTHORITY = 'cpan:APOCAL';
@@ -40,33 +40,46 @@ has 'check_cpan' => (
 {
 	use Moose::Util::TypeConstraints 1.01;
 
+
+	# TODO add CPANIDX?
 	has 'cpan_backend' => (
 		is	=> 'rw',
 		isa	=> enum( [ qw( CPANPLUS CPAN CPANSQLite ) ] ),
 		default	=> 'CPANPLUS',
-		trigger => \&_clean_backend,
+		trigger => \&_clean_cpan_backend,
 	);
 
-	sub _clean_backend {
+	sub _clean_cpan_backend {
 		my( $self, $new, $old ) = @_;
 
-		# Just clear the cpan backend
+		# Just clear the cpan cache
 		$self->_cache->{'cpan'} = {};
+		$self->_backend_err( 0 );
 	}
 }
 
+
+has 'cpan_backend_auto' => (
+	is	=> 'rw',
+	isa	=> 'Bool',
+	default	=> 1,
+);
+
+
 has 'cpan_section_err' => (
 	is	=> 'rw',
 	isa	=> 'Bool',
 	default	=> 0,
 );
 
+
 has 'verbose' => (
 	is	=> 'rw',
 	isa	=> 'Bool',
-	default	=> 1,
+	default	=> 0,
 );
 
+# holds the cached results of link look-ups
 has '_cache' => (
 	is	=> 'ro',
 	isa	=> 'HashRef',
@@ -78,6 +91,23 @@ has '_cache' => (
 	} },
 );
 
+# is the backend good to use?
+has '_backend_err' => (
+	is	=> 'rw',
+	isa	=> 'Bool',
+	default	=> 0,
+	trigger => \&_clean_backend_err,
+);
+
+sub _clean_backend_err {
+	my( $self, $new, $old ) = @_;
+
+	# Only clean if an error happened
+	if ( $new ) {
+		$self->_cache->{'cpan'} = {};
+	}
+}
+
 
 sub pod_ok {
 	my $self = shift;
@@ -122,9 +152,9 @@ sub pod_ok {
 
 			# TODO ugly, but there is no other way to get at it?
 			## no critic ( ProhibitAccessOfPrivateData )
-			foreach my $l ( keys %{ $parser->{errata} } ) {
+			foreach my $l ( keys %{ $parser->{'errata'} } ) {
 				$Test->diag( " * errors seen in line $l:" );
-				$Test->diag( "   * $_" ) for @{ $parser->{errata}{$l} };
+				$Test->diag( "   * $_" ) for @{ $parser->{'errata'}{$l} };
 			}
 		}
 
@@ -197,13 +227,13 @@ sub _analyze {
 	foreach my $l ( @$links ) {
 		## no critic ( ProhibitAccessOfPrivateData )
 		my( $type, $to, $section, $linenum, $column ) = @$l;
-		push( @diag, "$file:$linenum:$column - Checking link '$type/" . ( defined $to ? $to : '' ) . "/" .
-			( defined $section ? $section : '' ) . "'" ) if $ENV{'TEST_VERBOSE'};
+		push( @diag, "$file:$linenum:$column - Checking link type($type) to(" . ( defined $to ? $to : '' ) . ") " .
+			"section(" . ( defined $section ? $section : '' ) . ")" ) if $self->verbose;
 
 		# What kind of link?
 		if ( $type eq 'man' ) {
 			if ( ! $self->_known_manpage( $to ) ) {
-				push( @errors, "$file:$linenum:$column - Unknown manpage '$to'" );
+				push( @errors, "$file:$linenum:$column - Unknown link type(man) to($to)" );
 			}
 		} elsif ( $type eq 'pod' ) {
 			# do we have a to/section?
@@ -217,13 +247,15 @@ sub _analyze {
 							if ( $res ) {
 								# if true, treat cpan sections as errors because we can't verify if section exists
 								if ( $self->cpan_section_err ) {
-									push( @errors, "$file:$linenum:$column - Unable to verify pod link '$to/$section' because the CPAN module is not installed" );
+									push( @errors, "$file:$linenum:$column - Unable to verify link type(pod) to($to) section($section) because the module isn't installed" );
+								} else {
+									push( @diag, "$file:$linenum:$column - Unable to verify link type(pod) to($to) section($section) because the module isn't installed" );
 								}
 							} else {
-								push( @errors, "$file:$linenum:$column - Unknown pod link '$to/$section' - module doesn't exist in CPAN" );
+								push( @errors, "$file:$linenum:$column - Unknown link type(pod) to($to) section($section) - module doesn't exist in CPAN" );
 							}
 						} else {
-							push( @errors, "$file:$linenum:$column - Unknown pod link '$to/$section' - unable to check CPAN" );
+							push( @errors, "$file:$linenum:$column - Unknown link type(pod) to($to) section($section) - unable to check CPAN" );
 						}
 					}
 				} else {
@@ -237,17 +269,17 @@ sub _analyze {
 								if ( ! $res ) {
 									# Check for internal section
 									if ( exists $own_sections->{ $to } ) {
-										push( @diag, "$file:$linenum:$column - Internal section link - recommend 'L</$to>'" );
+										push( @diag, "$file:$linenum:$column - Link type(pod) to($to) looks like an internal section link - recommend 'L</$to>'" );
 									} else {
-										push( @errors, "$file:$linenum:$column - Unknown pod file '$to' - module doesn't exist in CPAN" );
+										push( @errors, "$file:$linenum:$column - Unknown link type(pod) to($to) - module doesn't exist in CPAN" );
 									}
 								}
 							} else {
 								# Check for internal section
 								if ( exists $own_sections->{ $to } ) {
-									push( @diag, "$file:$linenum:$column - Internal section link - recommend 'L</$to>'" );
+									push( @diag, "$file:$linenum:$column - Link type(pod) to($to) looks like an internal section link - recommend 'L</$to>'" );
 								} else {
-									push( @errors, "$file:$linenum:$column - Unknown pod link '$to' - unable to check CPAN" );
+									push( @errors, "$file:$linenum:$column - Unknown link type(pod) to($to) - unable to check CPAN" );
 								}
 							}
 						}
@@ -256,15 +288,16 @@ sub _analyze {
 			} else {
 				if ( defined $section ) {
 					if ( ! exists $own_sections->{ $section } ) {
-						push( @errors, "$file:$linenum:$column - Unknown local pod section '$section'" );
+						push( @errors, "$file:$linenum:$column - Unknown link type(pod) to() section($section) - section doesn't exist!" );
 					}
 				} else {
 					# no to/section eh?
-					die "Invalid link: $l";
+					push( @errors, "$file:$linenum:$column - Malformed link type(pod) to() section()" );
 				}
 			}
 		} else {
-			die "Unknown type: $type";
+			# unknown type?
+			push( @errors, "$file:$linenum:$column - Unknown link type($type) to(" . ( defined $to ? $to : '' ) . ") section(" . ( defined $section ? $section : '' ) . ")" );
 		}
 	}
 
@@ -342,6 +375,16 @@ sub _known_cpan {
 		return undef;
 	}
 
+	# Did the backend encounter an error?
+	if ( $self->_backend_err ) {
+		return undef;
+	}
+
+	# Sanity check - we use '.' as the actual cache placeholder...
+	if ( $module eq '.' ) {
+		return undef;
+	}
+
 	# is the answer cached already?
 	if ( exists $self->_cache->{'cpan'}{ $module } ) {
 		return $self->_cache->{'cpan'}{ $module };
@@ -385,54 +428,30 @@ sub _known_cpan_cpanplus {
 			$cache->{'.'} = CPANPLUS::Backend->new( $cpanconfig );
 		};
 		if ( $@ ) {
-			warn "Unable to load CPANPLUS - switching to CPAN ( $@ )" if $self->verbose;
-			$self->cpan_backend( 'CPAN' );
-			return $self->_known_cpan( $module );
+			warn "Unable to load CPANPLUS - $@" if $self->verbose;
+			if ( $self->cpan_backend_auto ) {
+				$self->cpan_backend( 'CPAN' );
+				return $self->_known_cpan_cpan( $module );
+			} else {
+				$self->_backend_err( 1 );
+				return undef;
+			}
 		}
 	}
 
 	my $result = undef;
 	eval { local $SIG{'__WARN__'} = sub { return }; $result = $cache->{'.'}->parse_module( 'module' => $module ) };
-	if ( ! $@ and defined $result ) {
-		$cache->{ $module } = 1;
-	} else {
-		$cache->{ $module } = 0;
-	}
-
-	return $cache->{ $module };
-}
-
-sub _known_cpan_cpansqlite {
-	my( $self, $module ) = @_;
-	my $cache = $self->_cache->{'cpan'};
-
-	# init the backend ( and set some options )
-	if ( ! exists $cache->{'.'} ) {
-		eval {
-			# Wacky format so dzil will not autoprereq it
-			require 'CPAN.pm'; require 'CPAN/SQLite.pm';
-
-			# TODO this code stolen from App::PodLinkCheck
-			# not sure how far back this will work, maybe only 5.8.0 up
-			if ( ! $CPAN::Config_loaded && CPAN::HandleConfig->can( 'load' ) ) {
-				# fake $loading to avoid running the CPAN::FirstTime dialog -- is this the right way to do that?
-				local $CPAN::HandleConfig::loading = 1;
-				CPAN::HandleConfig->load;
-			}
-
-			$cache->{'.'} = CPAN::SQLite->new;
-		};
-		if ( $@ ) {
-			$self->check_cpan( 0 );
-			delete $cache->{'.'} if exists $cache->{'.'};
-			warn "Unable to load CPANSQLite - disabling CPAN searches! ( $@ )" if $self->verbose;
+	if ( $@ ) {
+		warn "Unable to use CPANPLUS - $@" if $self->verbose;
+		if ( $self->cpan_backend_auto ) {
+			$self->cpan_backend( 'CPAN' );
+			return $self->_known_cpan_cpan( $module );
+		} else {
+			$self->_backend_err( 1 );
 			return undef;
 		}
 	}
-
-	my $result = undef;
-	eval { local $SIG{'__WARN__'} = sub { return }; $result = $cache->{'.'}->query( 'mode' => 'module', name => $module, max_results => 1 ) };
-	if ( ! $@ and defined $result ) {
+	if ( defined $result ) {
 		$cache->{ $module } = 1;
 	} else {
 		$cache->{ $module } = 0;
@@ -481,9 +500,14 @@ sub _known_cpan_cpan {
 			$cache->{'.'} = $CPAN::META->{'readwrite'}->{'CPAN::Module'};
 		};
 		if ( $@ ) {
-			warn "Unable to load CPAN - switching to CPANSQLite ( $@ )" if $self->verbose;
-			$self->cpan_backend( 'CPANSQLite' );
-			return $self->_known_cpan( $module );
+			warn "Unable to load CPAN - $@" if $self->verbose;
+			if ( $self->cpan_backend_auto ) {
+				$self->cpan_backend( 'CPANSQLite' );
+				return $self->_known_cpan_cpansqlite( $module );
+			} else {
+				$self->_backend_err( 1 );
+				return undef;
+			}
 		}
 	}
 
@@ -496,6 +520,61 @@ sub _known_cpan_cpan {
 	return $cache->{ $module };
 }
 
+sub _known_cpan_cpansqlite {
+	my( $self, $module ) = @_;
+	my $cache = $self->_cache->{'cpan'};
+
+	# init the backend ( and set some options )
+	if ( ! exists $cache->{'.'} ) {
+		eval {
+			# Wacky format so dzil will not autoprereq it
+			require 'CPAN.pm'; require 'CPAN/SQLite.pm';
+
+			# TODO this code stolen from App::PodLinkCheck
+			# not sure how far back this will work, maybe only 5.8.0 up
+			if ( ! $CPAN::Config_loaded && CPAN::HandleConfig->can( 'load' ) ) {
+				# fake $loading to avoid running the CPAN::FirstTime dialog -- is this the right way to do that?
+				local $CPAN::HandleConfig::loading = 1;
+				CPAN::HandleConfig->load;
+			}
+
+			$cache->{'.'} = CPAN::SQLite->new;
+		};
+		if ( $@ ) {
+			warn "Unable to load CPANSQLite - $@" if $self->verbose;
+			if ( $self->cpan_backend_auto ) {
+				warn "Unable to use any CPAN backend, disabling searches!" if $self->verbose;
+				$self->check_cpan( 0 );
+				$self->cpan_section_err( 1 );
+			}
+
+			$self->_backend_err( 1 );
+			return undef;
+		}
+	}
+
+	my $result = undef;
+	eval { local $SIG{'__WARN__'} = sub { return }; $result = $cache->{'.'}->query( 'mode' => 'module', name => $module, max_results => 1 ); };
+	if ( $@ ) {
+		warn "Unable to use CPANSQLite - $@" if $self->verbose;
+		if ( $self->cpan_backend_auto ) {
+			warn "Unable to use any CPAN backend, disabling searches!" if $self->verbose;
+			$self->check_cpan( 0 );
+			$self->cpan_section_err( 1 );
+		}
+
+		$self->_backend_err( 1 );
+		return undef;
+	}
+	if ( $result ) {
+		$cache->{ $module } = 1;
+	} else {
+		$cache->{ $module } = 0;
+	}
+
+	return $cache->{ $module };
+}
+
 sub _known_podlink {
 	## no critic ( ProhibitAccessOfPrivateData )
 	my( $self, $link, $section ) = @_;
@@ -547,7 +626,7 @@ Test::Pod::LinkCheck - Tests POD for invalid links
 
 =head1 VERSION
 
-  This document describes v0.004 of Test::Pod::LinkCheck - released May 27, 2010 as part of Test-Pod-LinkCheck.
+  This document describes v0.005 of Test::Pod::LinkCheck - released February 21, 2011 as part of Test-Pod-LinkCheck.
 
 =head1 SYNOPSIS
 
@@ -567,8 +646,7 @@ Test::Pod::LinkCheck - Tests POD for invalid links
 
 This module looks for any links in your POD and verifies that they point to a valid resource. It uses the L<Pod::Simple> parser
 to analyze the pod files and look at their links. In a nutshell, it looks for C<LE<lt>FooE<gt>> links and makes sure that Foo
-exists. It also recognizes section links, C<LE<lt>/SYNOPSISE<gt>> for example. Also, manpages are resolved and checked. If you
-linked to a CPAN module and it is not installed, it is an error!
+exists. It also recognizes section links, C<LE<lt>/SYNOPSISE<gt>> for example. Also, manpages are resolved and checked.
 
 This module does B<NOT> check "http" links like C<LE<lt>http://www.google.comE<gt>> in your pod. For that, please check
 out L<Test::Pod::No404s>.
@@ -589,8 +667,10 @@ C<pod_ok> function to be imported when you use this module.
 
 =head2 check_cpan
 
-If disabled, this module will not check the CPAN module database to see if a link is a valid CPAN module or not. As of now
-this module only supports L<CPANPLUS> as the backend, others may be added.
+If enabled, this module will check the CPAN module database to see if a link is a valid CPAN module or not. It uses the backend
+defined in L</cpan_backend> to do the actual searching.
+
+If disabled, it will resolve links based on locally installed modules. If it isn't installed it will be an error!
 
 The default is: true
 
@@ -600,10 +680,22 @@ Selects the CPAN backend to use for querying modules. The available ones are: CP
 
 The default is: CPANPLUS
 
+	The backends were tested and verified against those versions. Older versions would work, but is untested!
+		CPANPLUS v0.9010
+		CPAN v1.9402
+		CPAN::SQLite v0.199
+
+=head2 cpan_backend_auto
+
+Enable to automatically try the CPAN backends to find an available one. It will try the backends in the order defined in L</cpan_backend>
+
+If no backend is available, it will disable the L</check_cpan> attribute and enable the L</cpan_section_err> attribute.
+
+The default is: true
+
 =head2 cpan_section_err
 
-If enabled, a link pointing to a CPAN module's specific section is treated as an error. Since the module isn't installed we
-are unable to verify the section actually exists.
+If enabled, a link pointing to a CPAN module's specific section is treated as an error if it isn't installed.
 
 The default is: false
 
@@ -611,7 +703,7 @@ The default is: false
 
 If enabled, this module will print extra diagnostics for the links it's checking.
 
-The default is: true
+The default is: false
 
 =head1 METHODS
 
@@ -633,30 +725,56 @@ Accepts an optional array of files to check. By default it uses all POD files in
 This method is what you will usually run. Every file is passed to the L</pod_ok> function. This also sets the
 test plan to be the number of files.
 
+=head1 NOTES
+
+=head2 backend
+
+This module uses the L<CPANPLUS> and L<CPAN> modules as the backend to verify valid CPAN modules. Please make sure that the backend you
+choose is properly configured before running this! This means the index is updated, permissions is set, and whatever else the backend
+needs to properly function. If you don't want to use them please disable the L</check_cpan> attribute.
+
+If this module fails to check CPAN modules or the testsuite fails, it's probably because of the above reason.
+
+=head2 CPAN module sections
+
+One limitation of this module is that it can't check for valid sections on CPAN modules if they aren't installed. If you want that to be an
+error, please enable the L</cpan_section_err> attribute.
+
 =head1 SEE ALSO
+
+Please see those modules/websites for more information related to this module.
+
+=over 4
+
+=item *
+
 L<App::PodLinkCheck>
+
+=item *
+
 L<Pod::Checker>
+
+=item *
+
 L<Test::Pod::No404s>
 
-=begin stopwords
+=back
 
-CPAN
-AnnoCPAN
-RT
-CPANTS
-Kwalitee
-diff
-
-=end stopwords
+=for :stopwords cpan testmatrix url annocpan anno bugtracker rt cpants kwalitee diff irc mailto metadata placeholders
 
 =head1 SUPPORT
 
+=head2 Perldoc
+
 You can find documentation for this module with the perldoc command.
 
   perldoc Test::Pod::LinkCheck
 
 =head2 Websites
 
+The following websites have more information about this module, and may be of help to you. As always,
+in addition to those websites please use your favorite search engine to discover more resources.
+
 =over 4
 
 =item *
@@ -667,6 +785,12 @@ L<http://search.cpan.org/dist/Test-Pod-LinkCheck>
 
 =item *
 
+RT: CPAN's Bug Tracker
+
+L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Test-Pod-LinkCheck>
+
+=item *
+
 AnnoCPAN: Annotated CPAN documentation
 
 L<http://annocpan.org/dist/Test-Pod-LinkCheck>
@@ -685,12 +809,6 @@ L<http://cpanforum.com/dist/Test-Pod-LinkCheck>
 
 =item *
 
-RT: CPAN's Bug Tracker
-
-L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Test-Pod-LinkCheck>
-
-=item *
-
 CPANTS Kwalitee
 
 L<http://cpants.perl.org/dist/overview/Test-Pod-LinkCheck>
@@ -707,9 +825,48 @@ CPAN Testers Matrix
 
 L<http://matrix.cpantesters.org/?dist=Test-Pod-LinkCheck>
 
+=back
+
+=head2 Email
+
+You can email the author of this module at C<APOCAL at cpan.org> asking for help with any problems you have.
+
+=head2 Internet Relay Chat
+
+You can get live help by using IRC ( Internet Relay Chat ). If you don't know what IRC is,
+please read this excellent guide: L<http://en.wikipedia.org/wiki/Internet_Relay_Chat>. Please
+be courteous and patient when talking to us, as we might be busy or sleeping! You can join
+those networks/channels and get help:
+
+=over 4
+
 =item *
 
-Source Code Repository
+irc.perl.org
+
+You can connect to the server at 'irc.perl.org' and join this channel: #perl-help then talk to this person for help: Apocalypse.
+
+=item *
+
+irc.freenode.net
+
+You can connect to the server at 'irc.freenode.net' and join this channel: #perl then talk to this person for help: Apocal.
+
+=item *
+
+irc.efnet.org
+
+You can connect to the server at 'irc.efnet.org' and join this channel: #perl then talk to this person for help: Ap0cal.
+
+=back
+
+=head2 Bugs / Feature Requests
+
+Please report any bugs or feature requests by email to C<bug-test-pod-linkcheck at rt.cpan.org>, or through
+the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Test-Pod-LinkCheck>. You will be automatically notified of any
+progress on the request by the system.
+
+=head2 Source Code
 
 The code is open to the world, and available for you to hack on. Please feel free to browse it and play
 with it, or whatever. If you want to contribute patches, please send me a diff or prod me to pull
@@ -717,26 +874,20 @@ from your repository :)
 
 L<http://github.com/apocalypse/perl-test-pod-linkcheck>
 
-=back
-
-=head2 Bugs
-
-Please report any bugs or feature requests to C<bug-test-pod-linkcheck at rt.cpan.org>, or through
-the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Test-Pod-LinkCheck>.  I will be
-notified, and then you'll automatically be notified of progress on your bug as I make changes.
+  git clone git://github.com/apocalypse/perl-test-pod-linkcheck.git
 
 =head1 AUTHOR
 
-  Apocalypse <APOCAL@cpan.org>
+Apocalypse <APOCAL@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2010 by Apocalypse.
+This software is copyright (c) 2011 by Apocalypse.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
-The full text of the license can be found in the F<LICENSE> file included with this distribution.
+The full text of the license can be found in the LICENSE file included with this distribution.
 
 =cut
 
